import {Component, Input, OnInit} from '@angular/core';
import {Character} from "../../../project/game-model/characters/Character";
import {GameModel} from "../../../project/game-model/GameModel";
import {AbstractInteraction} from "../../../project/game-model/interactions/AbstractInteraction";
import {MatColumnDef, MatTable, MatTableDataSource} from "@angular/material/table";
import {InteractionSequences} from "../../../project/game-model/interactions/InteractionSequences";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {Interaction} from "../../../project/game-model/interactions/Interaction";
import {Condition} from "../../../project/game-model/interactions/condition/Condition";
import {MatSnackBar} from "@angular/material/snack-bar";

@Component({
  selector: 'app-character-interaction-editor',
  templateUrl: './character-interaction-editor.component.html',
  styleUrl: './character-interaction-editor.component.scss',
  animations: [
    trigger('detailExpand', [
      state('collapsed,void', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class CharacterInteractionEditorComponent implements OnInit{

  @Input() character: Character | undefined
  @Input() gameModel: GameModel | undefined

  displayedColumns: string[] = ['sequence', 'source', 'target', 'label', 'edit', 'delete']
  columnsToDisplayWithExpand = [... this.displayedColumns, 'expand'];
  expandedElement: AbstractInteraction | null = null;
  editedElement: AbstractInteraction | null = null;

  interactionDatasource: MatTableDataSource<AbstractInteraction> = new MatTableDataSource();

  constructor(private snackbar: MatSnackBar) {
  }


  ngOnInit() {
    this.interactionDatasource.data = this.gameModel!.getCharacterInteractionsByCharacter(this.character!);
  }

  isInteractionSequence(interaction: AbstractInteraction) {
    return interaction instanceof InteractionSequences
  }

  addInteraction() {
    const interaction = new Interaction(this.character!, undefined, "")
    this.editedElement = interaction;

    const interactions = this.interactionDatasource.data;
    interactions.push(interaction);
    this.interactionDatasource.data = interactions;
  }

  submitInteraction() {
    if(this.editedElement == undefined) {
      return;
    }

    if(this.editedElement!.validate(this.character!)) {
      this.gameModel!.addCharacterInteraction(this.editedElement);
      this.editedElement = null;
    } else {
      this.snackbar.open("Invalid Interaction", "", {duration: 2000});
    }
  }

  deleteInteraction(interaction: AbstractInteraction) {
    this.gameModel!.removeCharacterInteraction(interaction);
    this.interactionDatasource.data = this.gameModel!.characterInteractions;
  }

  editInteraction(interaction: AbstractInteraction) {
    this.editedElement = interaction;
  }
}
