import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {AsyncPipe} from "@angular/common";
import {FormControl, FormsModule} from "@angular/forms";
import {MatAutocomplete, MatOption} from "@angular/material/autocomplete";
import {MatFormField, MatLabel} from "@angular/material/form-field";
import {MatInput} from "@angular/material/input";
import {map, Observable, startWith} from "rxjs";
import {ItemGroupCharacteristic} from "../../../../../project/game-model/inventory/ItemgroupCharacteristic";

@Component({
  selector: 'app-characteristic-selector',
  templateUrl: './characteristic-selector.component.html',
  styleUrl: './characteristic-selector.component.scss'
})
export class CharacteristicSelectorComponent implements OnInit{
  myControl = new FormControl('');
  @Input() options: ItemGroupCharacteristic[] = []
  @Input() selectedInput: ItemGroupCharacteristic | undefined
  filteredOptions: Observable<ItemGroupCharacteristic[]>;

  @Output('onSelectOption') selectedOptionEmitter: EventEmitter<ItemGroupCharacteristic> = new EventEmitter<ItemGroupCharacteristic>();

  constructor() {
    this.filteredOptions = this.myControl.valueChanges.pipe(
      startWith(''),
      map(state => (state ? this._filterStates(state) : this.options.slice())),
    );
  }

  ngOnInit() {
    if(this.selectedInput != undefined) {
      this.myControl.setValue(this.selectedInput.characteristicName);
    }
  }

  private _filterStates(value: any): ItemGroupCharacteristic[] {
    const filterValue = value.toLowerCase();

    return this.options.filter(state => state.characteristicName.toLowerCase().includes(filterValue));
  }

  onOptionSelected() {
    const selectedOption = this.options.find(characteristic => characteristic.characteristicName.toLowerCase() === this.myControl.value!.toLowerCase());
    this.selectedOptionEmitter.emit(selectedOption!)
  }
}
