import {Component, Input, OnInit} from '@angular/core';
import {InventorySlot} from "../../../../project/game-model/inventory/intentory-slots/InventorySlot";
import {ItemGroupCharacteristic} from "../../../../project/game-model/inventory/ItemgroupCharacteristic";
import {ItemGroup} from "../../../../project/game-model/inventory/ItemGroup";
import {MatTable, MatTableDataSource} from "@angular/material/table";
import {
  InventoryCharacteristic
} from "../../../../project/game-model/inventory/intentory-slots/InventoryCharacteristic";
import {MatSnackBar} from "@angular/material/snack-bar";

@Component({
  selector: 'app-inventory-slot-characteristic-editor',
  templateUrl: './inventory-slot-characteristic-editor.component.html',
  styleUrl: './inventory-slot-characteristic-editor.component.scss'
})
export class InventorySlotCharacteristicEditorComponent implements OnInit{

  @Input() inventorySlot: InventorySlot | undefined
  @Input() itemgroups: ItemGroup[] = []

  availableCharacteristics: ItemGroupCharacteristic[] = []

  datasource: MatTableDataSource<InventoryCharacteristic> = new MatTableDataSource<InventoryCharacteristic>();
  displayedColumns: string[] = ['increasing', 'decreasing', 'edit', 'delete']
  editedCharacteristic: InventoryCharacteristic | null = null;

  constructor(private snackbar: MatSnackBar) {
  }

  ngOnInit() {
    this.itemgroups.forEach(itemgroup => {
      itemgroup.itemGroupCharacteristics.forEach(characteristic => {
        if(!this.availableCharacteristics.includes(characteristic)) {
          this.availableCharacteristics.push(characteristic)
        }
      })
    })
    this.datasource.data = this.inventorySlot!.slotCharacteristics;
  }

  addInventorySlotCharacteristic() {
    const slotCharacteristic = new InventoryCharacteristic(null, null);
    this.inventorySlot!.addSlotCharacteristic(slotCharacteristic);
    this.datasource.data = this.inventorySlot!.slotCharacteristics;
    this.editedCharacteristic = slotCharacteristic;
  }

  finishEditing() {
    if(this.editedCharacteristic != null && this.editedCharacteristic.isValid()) {
      this.editedCharacteristic = null;
    } else if(this.editedCharacteristic != null) {
      this.snackbar.open("Please select a valid increasing and decreasing Characterstic", "", {duration: 2000});
    }
  }

  onSelectIncreasingCharacteristic(selectedCharacteristic: ItemGroupCharacteristic, slotCharacteristic: InventoryCharacteristic) {
    slotCharacteristic.increasingCharacteristic = selectedCharacteristic;
  }

  onSelectDecreasingCharacteristic(selectedCharacteristic: ItemGroupCharacteristic, slotCharacteristic: InventoryCharacteristic) {
    slotCharacteristic.decreasingCharacteristic = selectedCharacteristic;
  }

  editSlotCharacteristic(slotCharacteristic: InventoryCharacteristic) {
    this.editedCharacteristic = slotCharacteristic;
  }

  deleteSlotCharacteristic(slotCharacteristic: InventoryCharacteristic) {
    this.inventorySlot!.slotCharacteristics = this.inventorySlot!.slotCharacteristics.filter(characteristic => characteristic !== slotCharacteristic);
    this.datasource.data = this.inventorySlot!.slotCharacteristics;
  }
}
