import {Component, Input, OnInit} from '@angular/core';
import {Character} from "../../../project/game-model/characters/Character";
import {ItemGroup} from "../../../project/game-model/inventory/ItemGroup";
import {InventorySlot} from "../../../project/game-model/inventory/intentory-slots/InventorySlot";
import {MatDialog} from "@angular/material/dialog";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {MatTableDataSource} from "@angular/material/table";

@Component({
  selector: 'app-inventory-slot-editor',
  templateUrl: './inventory-slot-editor.component.html',
  styleUrl: './inventory-slot-editor.component.scss',
  animations: [
    trigger('detailExpand', [
      state('collapsed,void', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class InventorySlotEditorComponent implements OnInit{

  @Input() character: Character | undefined
  @Input() itemgroups: ItemGroup[] = []
  displayedColumns: string[] = ['name','combinable', 'edit', 'delete']
  columnsToDisplayWithExpand = [...this.displayedColumns, 'expand'];
  dataSource: MatTableDataSource<InventorySlot> = new MatTableDataSource<InventorySlot>();

  expandedSlot: InventorySlot | null = null
  editedSlot: InventorySlot | null = null

  constructor(private dialog: MatDialog)  {
  }

  ngOnInit() {
    this.dataSource.data = this.character!.inventorySlots;
  }

  addInventorySlot() {
    this.editedSlot = new InventorySlot("New Inventory Slot");
    this.character!.addInventorySlot(this.editedSlot);
    this.dataSource.data = this.character!.inventorySlots;
  }

  editInventorySlot(inventorySlot: InventorySlot) {
    this.editedSlot = inventorySlot
  }

  deleteInventorySlot(inventorySlot: InventorySlot) {
    this.character!.removeInventorySlot(inventorySlot);
    this.dataSource.data = this.character!.inventorySlots;
  }

  finishEditing() {
    this.editedSlot = null;
  }
}
