import {Component, ElementRef, Input, OnInit, ViewChildren} from '@angular/core';
import {Character} from "../../project/game-model/characters/Character";
import {GameModel} from "../../project/game-model/GameModel";
import {Item} from "../../project/game-model/inventory/Item";
import {MatCard, MatCardContent, MatCardHeader} from "@angular/material/card";
import {ItemPropertyDescription} from "../../project/game-model/inventory/ItemPropertyDescription";
import {ItemPropertyEditorComponent} from "./item-property-editor/item-property-editor.component";
import {MatTableDataSource} from "@angular/material/table";

@Component({
  selector: 'app-inventory-item-editor',
  templateUrl: './inventory-item-editor.component.html',
  styleUrl: './inventory-item-editor.component.scss'
})
export class InventoryItemEditorComponent implements OnInit{

  @Input() item: Item | undefined;
  @Input() gameModel: GameModel | undefined;
  @ViewChildren('perQualityPropertyEditors') perQualityPropertyEditors: ItemPropertyEditorComponent[] = [];

  displayedColumns: string[] = ['name', 'description', 'edit', 'delete']
  editedProperty: ItemPropertyDescription | undefined

  datasource: MatTableDataSource<ItemPropertyDescription> = new MatTableDataSource<ItemPropertyDescription>()

  ngOnInit() {
    this.datasource.data = this.item!.perQualityProperties
  }

  deletePerQualityProperty(itemProperty: ItemPropertyDescription) {
    this.item!.removePerQualityProperty(itemProperty);
    this.datasource.data = this.item!.perQualityProperties
    this.perQualityPropertyEditors.forEach(component => component.updatedDisplayedData())
  }

  editPerQualityProperty(itemProperty: ItemPropertyDescription) {
    this.editedProperty = itemProperty
  }

  finishEditing() {
    if(this.editedProperty!.propertyName.length > 0) {
      this.item!.editPerQualityProperty(this.editedProperty!);

      this.editedProperty = undefined;
      this.perQualityPropertyEditors.forEach(component => component.updatedDisplayedData())
    }
  }

  addQualityProperty() {
    const qualityDescription = new ItemPropertyDescription("New Property", "")
    this.item!.addPerQualityProperty(qualityDescription);
    this.datasource.data = this.item!.perQualityProperties
    this.editedProperty = qualityDescription;

    this.perQualityPropertyEditors.forEach(component => component.updatedDisplayedData())

  }
}
