import {Interaction} from "./Interaction";
import {AbstractInteraction} from "./AbstractInteraction";
import {Character} from "../characters/Character";

export class InteractionSequences extends AbstractInteraction {

  rootInteraction: InteractionSequenceNode


  constructor(interaction: Interaction, interactionLabel: string) {
    super(interaction.sourceCharacter, interaction.targetCharacter, interactionLabel);
    this.rootInteraction = new InteractionSequenceNode(interaction, []);
  }

  equals(other: AbstractInteraction): boolean {
    if(!(other instanceof InteractionSequences)) {
      return false;
    }

    const equealCharacters = this.sourceCharacter === other.sourceCharacter && this.targetCharacter === other.targetCharacter;
    const equalLabels = this.interactionLabel === other.interactionLabel;
    const equalConditions = this.conditions.every(condition => other.conditions.includes(condition)) && other.conditions.every(condition => this.conditions.includes(condition));

    const equalSequenceTree = this.rootInteraction.equals(other.rootInteraction);

    return equealCharacters && equalLabels && equalConditions && equalSequenceTree;
  }

  validate(requieredCharacter: Character): boolean {
    const validCharacters = this.sourceCharacter == requieredCharacter || this.targetCharacter == requieredCharacter;
    const validLabel = this.interactionLabel.length > 0;
    const validSequenceTree = this.rootInteraction.validate(requieredCharacter)

    return validCharacters && validLabel && validSequenceTree;
  }


}

class InteractionSequenceNode {

  root: Interaction
  children: InteractionSequenceNode[] = [];

  equals(other: InteractionSequenceNode) {
    const equalsRoot = this.root.equals(other.root);
    const equalsChildren = this.children.every(child => other.children.includes(child) && other.children.every(child => this.children.includes(child)));

    return equalsRoot && equalsChildren;
  }

  validate(requiredCharacter: Character): boolean {
    const validateRoot = this.root.validate(requiredCharacter);
    let validateChildren = true;

    for(let i=0; i<this.children.length; i++) {
      if(!this.children[i].validate(requiredCharacter)) {
        validateChildren = false;
        break;
      }
    }

    return validateRoot && validateChildren;
  }


  constructor(root: Interaction, children: InteractionSequenceNode[]) {
    this.root = root;
    this.children = children;
  }
}
